@@ -5,15 +5,13 @@ import (
 	"path/filepath"
 
 	"github.com/sirupsen/logrus"
-
-	"gitlab.com/gitlab-org/gitlab-runner/network"
 )
 
 func GetDefaultConfigFile() string {
 	return filepath.Join(getDefaultConfigDirectory(), "config.toml")
 }
 
-func getDefaultCertificateDirectory() string {
+func GetDefaultCertificateDirectory() string {
 	return filepath.Join(getDefaultConfigDirectory(), "certs")
 }
 
@@ -25,6 +23,4 @@ func init() {
 			logrus.WithError(err).Fatal("Couldn't set CONFIG_FILE environment variable")
 		}
 	}
-
-	network.CertificateDirectory = getDefaultCertificateDirectory()
 }
