@@ -152,6 +152,22 @@ type RunCommand struct {
 	runnerWorkerProcessingFailure *prometheus.CounterVec
 }
 
+func NewRunCommand(n common.Network, apiRequestsCollector prometheus.Collector) cli.Command {
+	cmd := &RunCommand{
+		ServiceName:          defaultServiceName,
+		network:              n,
+		apiRequestsCollector: apiRequestsCollector,
+		prometheusLogHook:    prometheus_helper.NewLogHook(),
+		failuresCollector:    prometheus_helper.NewFailuresCollector(),
+		healthHelper:         newHealthHelper(),
+		buildsHelper:         newBuildsHelper(),
+		runAt:                runAt,
+		reloadConfigInterval: common.ReloadConfigInterval,
+	}
+
+	return common.NewCommand("run", "run multi runner service", cmd)
+}
+
 func (mr *RunCommand) log() *logrus.Entry {
 	config := mr.configfile.Config()
 	concurrent := 0
@@ -375,6 +391,12 @@ func (mr *RunCommand) reloadConfig() error {
 	config := mr.configfile.Config()
 	mr.healthHelper.healthy = nil
 	mr.log().Println("Configuration loaded")
+
+	// Warn about legacy /ci URL suffix in runner configurations
+	for _, runner := range config.Runners {
+		runner.WarnOnLegacyCIURL()
+	}
+
 	mr.checkConfigConcurrency(config)
 	if c, err := config.Masked(); err == nil {
 		mr.log().Debugln(helpers.ToYAML(c))
@@ -1500,24 +1522,6 @@ func (mr *RunCommand) Collect(ch chan<- prometheus.Metric) {
 	mr.runnerWorkerProcessingFailure.Collect(ch)
 }
 
-func init() {
-	apiRequestsCollector := network.NewAPIRequestsCollector()
-
-	cmd := &RunCommand{
-		ServiceName:          defaultServiceName,
-		network:              network.NewGitLabClientWithAPIRequestsCollector(apiRequestsCollector),
-		apiRequestsCollector: apiRequestsCollector,
-		prometheusLogHook:    prometheus_helper.NewLogHook(),
-		failuresCollector:    prometheus_helper.NewFailuresCollector(),
-		healthHelper:         newHealthHelper(),
-		buildsHelper:         newBuildsHelper(),
-		runAt:                runAt,
-		reloadConfigInterval: common.ReloadConfigInterval,
-	}
-
-	common.RegisterCommand("run", "run multi runner service", cmd)
-}
-
 func (mr *RunCommand) checkConfigConcurrency(config *common.Config) {
 	var warnings []string
 	var solutions []string
