@@ -96,6 +96,10 @@ type RegisterCommand struct {
 	common.RunnerConfig
 }
 
+func NewRegisterCommand(n common.Network) cli.Command {
+	return common.NewCommand("register", "register a new runner", newRegisterCommand(n))
+}
+
 type AccessLevel string
 
 const (
@@ -562,7 +566,7 @@ func getHostname() string {
 	return hostname
 }
 
-func newRegisterCommand() *RegisterCommand {
+func newRegisterCommand(n common.Network) *RegisterCommand {
 	return &RegisterCommand{
 		RunnerConfig: common.RunnerConfig{
 			Name: getHostname(),
@@ -578,7 +582,7 @@ func newRegisterCommand() *RegisterCommand {
 		},
 		Locked:    true,
 		Paused:    false,
-		network:   network.NewGitLabClient(),
+		network:   n,
 		timeNowFn: time.Now,
 	}
 }
@@ -596,7 +600,3 @@ func accessLevelValid(levels []AccessLevel, givenLevel AccessLevel) bool {
 
 	return false
 }
-
-func init() {
-	common.RegisterCommand("register", "register a new runner", newRegisterCommand())
-}
