@@ -8,7 +8,6 @@ import (
 
 	"gitlab.com/gitlab-org/gitlab-runner/commands/internal/configfile"
 	"gitlab.com/gitlab-org/gitlab-runner/common"
-	"gitlab.com/gitlab-org/gitlab-runner/network"
 )
 
 type ResetTokenCommand struct {
@@ -23,6 +22,12 @@ type ResetTokenCommand struct {
 	PAT        string `long:"pat" description:"Personal access token to use in lieu of runner's old authentication token"`
 }
 
+func NewResetTokenCommand(n common.Network) cli.Command {
+	return common.NewCommand("reset-token", "reset a runner's token", &ResetTokenCommand{
+		network: n,
+	})
+}
+
 func (c *ResetTokenCommand) resetAllRunnerTokens(cfg *common.Config) {
 	logrus.Warningln("Resetting all runner authentication tokens")
 	for _, r := range cfg.Runners {
@@ -95,9 +100,3 @@ func (c *ResetTokenCommand) Execute(_context *cli.Context) {
 
 	log.Println("Updated")
 }
-
-func init() {
-	common.RegisterCommand("reset-token", "reset a runner's token", &ResetTokenCommand{
-		network: network.NewGitLabClient(),
-	})
-}
