@@ -8,7 +8,6 @@ import (
 
 	"gitlab.com/gitlab-org/gitlab-runner/commands/internal/configfile"
 	"gitlab.com/gitlab-org/gitlab-runner/common"
-	"gitlab.com/gitlab-org/gitlab-runner/network"
 )
 
 type VerifyCommand struct {
@@ -20,6 +19,12 @@ type VerifyCommand struct {
 	DeleteNonExisting bool   `long:"delete" description:"Delete no longer existing runners?"`
 }
 
+func NewVerifyCommand(n common.Network) cli.Command {
+	return common.NewCommand("verify", "verify all registered runners", &VerifyCommand{
+		network: n,
+	})
+}
+
 //nolint:gocognit
 func (c *VerifyCommand) Execute(context *cli.Context) {
 	userModeWarning(true)
@@ -74,9 +79,3 @@ func (c *VerifyCommand) Execute(context *cli.Context) {
 	}
 	logrus.Println("Updated", c.ConfigFile)
 }
-
-func init() {
-	common.RegisterCommand("verify", "verify all registered runners", &VerifyCommand{
-		network: network.NewGitLabClient(),
-	})
-}
