@@ -113,7 +113,7 @@ const (
 var ErrSkipBuildStage = errors.New("skip build stage")
 
 type Build struct {
-	JobResponse `yaml:",inline"`
+	JobResponse `yaml:",inline" inputs:"expand"`
 
 	SystemInterrupt  chan os.Signal `json:"-" yaml:"-"`
 	RootDir          string         `json:"-" yaml:"-"`
@@ -158,6 +158,8 @@ type Build struct {
 	Referees         []referees.Referee
 	ArtifactUploader func(config JobCredentials, bodyProvider ContentProvider, options ArtifactsOptions) (UploadState, string)
 
+	ExecuteStepFn func(ctx context.Context, connector Connector, build *Build, trace JobTrace) error
+
 	urlHelper urlHelper
 
 	OnBuildStageStartFn OnBuildStageFn
@@ -404,6 +406,36 @@ func (b *Build) StartBuild(
 	return nil
 }
 
+func (b *Build) executeStepStage(ctx context.Context, connector Connector, buildStage BuildStage, trace JobTrace) error {
+	if ctx.Err() != nil {
+		return ctx.Err()
+	}
+
+	b.OnBuildStageStartFn.Call(buildStage)
+	defer b.OnBuildStageEndFn.Call(buildStage)
+
+	b.setCurrentStage(buildStage)
+	b.Log().WithField("build_stage", buildStage).Debug("Executing build stage")
+
+	section := helpers.BuildSection{
+		Name:        string(buildStage),
+		SkipMetrics: !b.JobResponse.Features.TraceSections,
+		Run: func() error {
+			msg := fmt.Sprintf(
+				"%s%s%s",
+				helpers.ANSI_BOLD_CYAN,
+				GetStageDescription(buildStage),
+				helpers.ANSI_RESET,
+			)
+			b.logger.Println(msg)
+
+			return b.ExecuteStepFn(ctx, connector, b, trace)
+		},
+	}
+
+	return section.Execute(&b.logger)
+}
+
 func (b *Build) executeStage(ctx context.Context, buildStage BuildStage, executor Executor) error {
 	if ctx.Err() != nil {
 		return ctx.Err()
@@ -538,12 +570,46 @@ func (b *Build) executeArchiveCache(ctx context.Context, state error, executor E
 	return b.executeStage(ctx, BuildStageArchiveOnFailureCache, executor)
 }
 
-//nolint:gocognit
 func (b *Build) executeScript(ctx context.Context, trace JobTrace, executor Executor) error {
 	// track job start and create referees
 	startTime := time.Now()
 	b.createReferees(executor)
 
+	// execute prepare scripts
+	err, cont := b.executePrepareScripts(ctx, executor)
+	if !cont {
+		return err
+	}
+
+	// execute user provided scripts
+	//nolint:nestif
+	if err == nil {
+		if b.UseNativeSteps() && b.ExecuteStepFn != nil {
+			connector, ok := executor.(Connector)
+			if ok {
+				err = b.executeStepStage(ctx, connector, BuildStage("step_"+StepNameRun), trace)
+			} else {
+				return ExecutorStepRunnerConnectNotSupported
+			}
+		} else {
+			err = b.executeUserScripts(ctx, trace, executor)
+		}
+	}
+
+	archiveCacheErr := b.executeArchiveCache(ctx, err, executor)
+
+	artifactUploadErr := b.executeUploadArtifacts(ctx, err, executor)
+
+	// track job end and execute referees
+	endTime := time.Now()
+	b.executeUploadReferees(ctx, startTime, endTime)
+
+	b.removeFileBasedVariables(ctx, executor)
+
+	return b.pickPriorityError(err, archiveCacheErr, artifactUploadErr)
+}
+
+func (b *Build) executePrepareScripts(ctx context.Context, executor Executor) (error, bool) {
 	// Prepare stage
 	err := b.executeStage(ctx, BuildStagePrepare, executor)
 	if err != nil {
@@ -551,7 +617,7 @@ func (b *Build) executeScript(ctx context.Context, trace JobTrace, executor Exec
 			"prepare environment: %w. "+
 				"Check https://docs.gitlab.com/runner/shells/#shell-profile-loading for more information",
 			err,
-		)
+		), false
 	}
 
 	err = b.attemptExecuteStage(ctx, BuildStageGetSources, executor, b.GetGetSourcesAttempts(), func(attempt int) error {
@@ -572,84 +638,77 @@ func (b *Build) executeScript(ctx context.Context, trace JobTrace, executor Exec
 		err = b.attemptExecuteStage(ctx, BuildStageDownloadArtifacts, executor, b.GetDownloadArtifactsAttempts(), nil)
 	}
 
-	//nolint:nestif
-	if err == nil {
-		timeouts := b.getStageTimeoutContexts(ctx,
-			stageTimeout{"RUNNER_SCRIPT_TIMEOUT", 0},
-			stageTimeout{"RUNNER_AFTER_SCRIPT_TIMEOUT", AfterScriptTimeout})
+	return err, true
+}
 
-		scriptCtx, cancel := timeouts["RUNNER_SCRIPT_TIMEOUT"]()
-		defer cancel()
+func (b *Build) executeUserScripts(ctx context.Context, trace JobTrace, executor Executor) error {
+	var err error
 
-		// update trace's cancel function so that the main script can be cancelled,
-		// with after_script and later stages to still complete.
-		trace.SetCancelFunc(cancel)
+	timeouts := b.getStageTimeoutContexts(ctx,
+		stageTimeout{"RUNNER_SCRIPT_TIMEOUT", 0},
+		stageTimeout{"RUNNER_AFTER_SCRIPT_TIMEOUT", AfterScriptTimeout})
 
-		b.printPolicyOptions()
+	scriptCtx, cancel := timeouts["RUNNER_SCRIPT_TIMEOUT"]()
+	defer cancel()
 
-		for _, s := range b.Steps {
-			// after_script has a separate BuildStage. See common.BuildStageAfterScript
-			if s.Name == StepNameAfterScript {
-				continue
-			}
-			err = b.executeStage(scriptCtx, StepToBuildStage(s), executor)
-			if err != nil {
-				break
-			}
+	// update trace's cancel function so that the main script can be cancelled,
+	// with after_script and later stages to still complete.
+	trace.SetCancelFunc(cancel)
+
+	b.printPolicyOptions()
+
+	for _, s := range b.Steps {
+		// after_script has a separate BuildStage. See common.BuildStageAfterScript
+		if s.Name == StepNameAfterScript {
+			continue
+		}
+		err = b.executeStage(scriptCtx, StepToBuildStage(s), executor)
+		if err != nil {
+			break
+		}
+	}
+
+	switch {
+	// if parent context is fine but script context was cancelled we ensure the build error
+	// failure reason is "canceled".
+	case ctx.Err() == nil && errors.Is(scriptCtx.Err(), context.Canceled):
+		err = &BuildError{
+			Inner:         ErrJobCanceled,
+			FailureReason: JobCanceled,
 		}
 
-		switch {
-		// if parent context is fine but script context was cancelled we ensure the build error
-		// failure reason is "canceled".
-		case ctx.Err() == nil && errors.Is(scriptCtx.Err(), context.Canceled):
-			err = &BuildError{
-				Inner:         ErrJobCanceled,
-				FailureReason: JobCanceled,
-			}
+		b.logger.Warningln("script canceled externally (UI, API)")
 
-			b.logger.Warningln("script canceled externally (UI, API)")
-
-		// If the parent context reached deadline, don't do anything different than usual.
-		// If the script context reached deadline, return the deadline error.
-		case !errors.Is(ctx.Err(), context.DeadlineExceeded) && errors.Is(scriptCtx.Err(), context.DeadlineExceeded):
-			err = &BuildError{
-				Inner:         fmt.Errorf("%w: %w", ErrJobScriptTimeout, scriptCtx.Err()),
-				FailureReason: JobExecutionTimeout,
-			}
+	// If the parent context reached deadline, don't do anything different than usual.
+	// If the script context reached deadline, return the deadline error.
+	case !errors.Is(ctx.Err(), context.DeadlineExceeded) && errors.Is(scriptCtx.Err(), context.DeadlineExceeded):
+		err = &BuildError{
+			Inner:         fmt.Errorf("%w: %w", ErrJobScriptTimeout, scriptCtx.Err()),
+			FailureReason: JobExecutionTimeout,
 		}
+	}
 
-		afterScriptCtx, cancel := timeouts["RUNNER_AFTER_SCRIPT_TIMEOUT"]()
-		defer cancel()
+	afterScriptCtx, cancel := timeouts["RUNNER_AFTER_SCRIPT_TIMEOUT"]()
+	defer cancel()
 
-		if afterScriptErr := b.executeAfterScript(afterScriptCtx, err, executor); afterScriptErr != nil {
-			// the parent deadline being exceeded is reported at a later stage, so we
-			// only focus on errors specific to after_script here.
-			if !errors.Is(ctx.Err(), context.DeadlineExceeded) {
-				// By default after-script ignores errors, but this can
-				// be disabled via the AFTER_SCRIPT_IGNORE_ERRORS variable.
+	if afterScriptErr := b.executeAfterScript(afterScriptCtx, err, executor); afterScriptErr != nil {
+		// the parent deadline being exceeded is reported at a later stage, so we
+		// only focus on errors specific to after_script here.
+		if !errors.Is(ctx.Err(), context.DeadlineExceeded) {
+			// By default after-script ignores errors, but this can
+			// be disabled via the AFTER_SCRIPT_IGNORE_ERRORS variable.
 
-				if b.Settings().AfterScriptIgnoreErrors {
-					b.logger.Warningln("after_script failed, but job will continue unaffected:", afterScriptErr)
-				} else if err == nil {
-					// If there's an existing error don't overwrite it with
-					// the after-script error.
-					err = afterScriptErr
-				}
+			if b.Settings().AfterScriptIgnoreErrors {
+				b.logger.Warningln("after_script failed, but job will continue unaffected:", afterScriptErr)
+			} else if err == nil {
+				// If there's an existing error don't overwrite it with
+				// the after-script error.
+				err = afterScriptErr
 			}
 		}
 	}
 
-	archiveCacheErr := b.executeArchiveCache(ctx, err, executor)
-
-	artifactUploadErr := b.executeUploadArtifacts(ctx, err, executor)
-
-	// track job end and execute referees
-	endTime := time.Now()
-	b.executeUploadReferees(ctx, startTime, endTime)
-
-	b.removeFileBasedVariables(ctx, executor)
-
-	return b.pickPriorityError(err, archiveCacheErr, artifactUploadErr)
+	return err
 }
 
 func (b *Build) pickPriorityError(jobErr error, archiveCacheErr error, artifactUploadErr error) error {
@@ -750,7 +809,7 @@ func (b *Build) attemptExecuteStage(
 	}
 
 	var err error
-	for attempt := 0; attempt < attempts; attempt++ {
+	for attempt := range attempts {
 		if retryCallback != nil {
 			if err = retryCallback(attempt); err != nil {
 				continue
@@ -761,6 +820,10 @@ func (b *Build) attemptExecuteStage(
 			return nil
 		}
 
+		if attempt == attempts-1 {
+			break
+		}
+
 		if b.IsFeatureFlagOn(featureflags.UseExponentialBackoffStageRetry) {
 			duration := retry.Duration()
 			b.logger.Infoln(fmt.Sprintf("Retrying in %v", duration))
@@ -1001,20 +1064,17 @@ func (b *Build) getTerminalTimeout(ctx context.Context, timeout time.Duration) t
 // If an error cannot be unwrapped to `BuildError`, `SystemFailure`
 // is used as the failure reason.
 func (b *Build) setTraceStatus(trace JobTrace, err error) {
-	logger := buildlogger.New(
-		trace, b.Log().WithFields(logrus.Fields{
-			"duration_s": b.FinalDuration().Seconds(),
-		}),
-		buildlogger.Options{
-			Timestamping:         b.IsFeatureFlagOn(featureflags.UseTimestamps),
-			MaskAllDefaultTokens: b.IsFeatureFlagOn(featureflags.MaskAllDefaultTokens),
-		},
-	)
-	defer logger.Close()
+	logger := b.Log().WithFields(logrus.Fields{
+		"duration_s": b.FinalDuration().Seconds(),
+	})
+
+	buildLogger := b.getNewLogger(trace, logger, true)
+	defer buildLogger.Close()
 
 	if err == nil {
-		logger.Infoln("Job succeeded")
-		logTerminationError(logger, "Success", trace.Success())
+		logger.WithFields(logrus.Fields{"job-status": "success"}).Infoln("Job succeeded")
+		buildLogger.Infoln("Job succeeded")
+		logTerminationError(buildLogger, "Success", trace.Success())
 
 		return
 	}
@@ -1025,25 +1085,40 @@ func (b *Build) setTraceStatus(trace JobTrace, err error) {
 	if errors.As(err, &buildError) {
 		b.failureReason = buildError.FailureReason
 
-		msg := fmt.Sprintln("Job failed:", err)
+		msg := fmt.Sprint("Job failed: ", err)
 		if buildError.FailureReason == RunnerSystemFailure {
-			msg = fmt.Sprintln("Job failed (system failure):", err)
+			msg = fmt.Sprint("Job failed (system failure): ", err)
 		}
 
-		logger.SoftErrorln(msg)
+		logger.
+			WithFields(logrus.Fields{
+				"job-status":     "failed",
+				"error":          err,
+				"failure_reason": buildError.FailureReason,
+				"exit_code":      buildError.ExitCode,
+			}).
+			Warningln(msg)
+		buildLogger.SoftErrorln(msg)
 
 		trace.SetSupportedFailureReasonMapper(newFailureReasonMapper(b.Features.FailureReasons))
 		err = trace.Fail(err, JobFailureData{
 			Reason:   buildError.FailureReason,
 			ExitCode: buildError.ExitCode,
 		})
-		logTerminationError(logger, "Fail", err)
+		logTerminationError(buildLogger, "Fail", err)
 
 		return
 	}
 
-	logger.Errorln("Job failed (system failure):", err)
-	logTerminationError(logger, "Fail", trace.Fail(err, JobFailureData{Reason: RunnerSystemFailure}))
+	logger.
+		WithFields(logrus.Fields{
+			"job-status":     "failed",
+			"error":          err,
+			"failure_reason": RunnerSystemFailure,
+		}).
+		Errorln("Job failed (system failure):", err)
+	buildLogger.Errorln("Job failed (system failure):", err)
+	logTerminationError(buildLogger, "Fail", trace.Fail(err, JobFailureData{Reason: RunnerSystemFailure}))
 }
 
 func logTerminationError(logger buildlogger.Logger, name string, err error) {
@@ -1090,6 +1165,11 @@ func (b *Build) Run(globalConfig *Config, trace JobTrace) (err error) {
 		}
 	}()
 
+	err = b.expandInputs()
+	if err != nil {
+		return fmt.Errorf("failed to expand inputs: %w", err)
+	}
+
 	b.logUsedImages()
 	b.printRunningWithHeader(trace)
 
@@ -1100,16 +1180,7 @@ func (b *Build) Run(globalConfig *Config, trace JobTrace) (err error) {
 
 	b.expandContainerOptions()
 
-	b.logger = buildlogger.New(
-		trace,
-		b.Log(),
-		buildlogger.Options{
-			MaskPhrases:          b.GetAllVariables().Masked(),
-			MaskTokenPrefixes:    b.JobResponse.Features.TokenMaskPrefixes,
-			Timestamping:         b.IsFeatureFlagOn(featureflags.UseTimestamps),
-			MaskAllDefaultTokens: b.IsFeatureFlagOn(featureflags.MaskAllDefaultTokens),
-		},
-	)
+	b.logger = b.getNewLogger(trace, b.Log(), false)
 	defer b.logger.Close()
 
 	ctx, cancel := context.WithTimeout(context.Background(), b.GetBuildTimeout())
@@ -1151,6 +1222,37 @@ func (b *Build) Run(globalConfig *Config, trace JobTrace) (err error) {
 	return err
 }
 
+// expandInputs expands inputs in various build configuration settings.
+//
+// TODO: we want to expand inputs as early as possible to optimize the feedback loop.
+// However, that may lead to problems where certain expansion context is only available later on.
+// This might not be a problem for Inputs itself, but for functions (like `now()`) or
+// when we allow other context in the expression, like access to environment variables,
+// or other job-runtime dependent features.
+// For a good middle ground we could parse the scripts as moa expressions and cache them
+// and only later on evaluate given the necessary context.
+func (b *Build) expandInputs() error {
+	if !b.IsFeatureFlagOn(featureflags.EnableJobInputsInterpolation) {
+		return nil
+	}
+
+	return ExpandInputs(&b.Inputs, b)
+}
+
+func (b *Build) getNewLogger(trace JobTrace, log *logrus.Entry, teeOnly bool) buildlogger.Logger {
+	return buildlogger.New(
+		trace,
+		log,
+		buildlogger.Options{
+			MaskPhrases:          b.GetAllVariables().Masked(),
+			MaskTokenPrefixes:    b.JobResponse.Features.TokenMaskPrefixes,
+			Timestamping:         b.IsFeatureFlagOn(featureflags.UseTimestamps),
+			MaskAllDefaultTokens: b.IsFeatureFlagOn(featureflags.MaskAllDefaultTokens),
+			TeeOnly:              teeOnly,
+		},
+	)
+}
+
 func (b *Build) logUsedImages() {
 	if !b.IsFeatureFlagOn(featureflags.LogImagesConfiguredForJob) {
 		return
@@ -1195,14 +1297,7 @@ func (b *Build) resolveSecrets(trace JobTrace) error {
 		Name:        string(BuildStageResolveSecrets),
 		SkipMetrics: !b.JobResponse.Features.TraceSections,
 		Run: func() error {
-			logger := buildlogger.New(
-				trace,
-				b.Log(),
-				buildlogger.Options{
-					Timestamping:         b.IsFeatureFlagOn(featureflags.UseTimestamps),
-					MaskAllDefaultTokens: b.IsFeatureFlagOn(featureflags.MaskAllDefaultTokens),
-				},
-			)
+			logger := b.getNewLogger(trace, b.Log(), false)
 			defer logger.Close()
 
 			resolver, err := b.secretsResolver(&logger, GetSecretResolverRegistry(), b.IsFeatureFlagOn)
@@ -1346,29 +1441,29 @@ func (b *Build) GetSharedEnvVariable() JobVariable {
 func (b *Build) GetCITLSVariables() JobVariables {
 	variables := JobVariables{}
 
-	if b.TLSCAChain != "" {
+	if b.TLSData.CAChain != "" {
 		variables = append(variables, JobVariable{
 			Key:      tls.VariableCAFile,
-			Value:    b.TLSCAChain,
+			Value:    b.TLSData.CAChain,
 			Public:   true,
 			Internal: true,
 			File:     true,
 		})
 	}
 
-	if b.TLSAuthCert != "" && b.TLSAuthKey != "" {
+	if b.TLSData.AuthCert != "" && b.TLSData.AuthKey != "" {
 		variables = append(
 			variables,
 			JobVariable{
 				Key:      tls.VariableCertFile,
-				Value:    b.TLSAuthCert,
+				Value:    b.TLSData.AuthCert,
 				Public:   true,
 				Internal: true,
 				File:     true,
 			},
 			JobVariable{
 				Key:      tls.VariableKeyFile,
-				Value:    b.TLSAuthKey,
+				Value:    b.TLSData.AuthKey,
 				Internal: true,
 				File:     true,
 			},
@@ -1562,13 +1657,13 @@ func (b *Build) withUrlHelper() urlHelper {
 }
 
 // GetRemoteURL uses the urlHelper to get the remote URL used for fetching the repo.
-func (b *Build) GetRemoteURL() (string, error) {
+func (b *Build) GetRemoteURL() (*url.URL, error) {
 	return b.withUrlHelper().GetRemoteURL()
 }
 
-// GetURLInsteadOfArgs uses the urlHelper to generate insteadOf URLs to pass on to git.
-func (b *Build) GetURLInsteadOfArgs() ([]string, error) {
-	return b.withUrlHelper().GetURLInsteadOfArgs()
+// GetInsteadOfs uses the urlHelper to generate insteadOf URLs to pass on to git.
+func (b *Build) GetInsteadOfs() ([][2]string, error) {
+	return b.withUrlHelper().GetInsteadOfs()
 }
 
 type stageTimeout struct {
@@ -1747,8 +1842,8 @@ func (b *Build) ensureFinishedAt() {
 }
 
 type urlHelper interface {
-	GetRemoteURL() (string, error)
-	GetURLInsteadOfArgs() ([]string, error)
+	GetRemoteURL() (*url.URL, error)
+	GetInsteadOfs() ([][2]string, error)
 }
 
 func NewBuild(
@@ -1794,14 +1889,7 @@ func (b *Build) getFeatureFlagInfo() string {
 }
 
 func (b *Build) printRunningWithHeader(trace JobTrace) {
-	logger := buildlogger.New(
-		trace,
-		b.Log(),
-		buildlogger.Options{
-			Timestamping:         b.IsFeatureFlagOn(featureflags.UseTimestamps),
-			MaskAllDefaultTokens: b.IsFeatureFlagOn(featureflags.MaskAllDefaultTokens),
-		},
-	)
+	logger := b.getNewLogger(trace, b.Log(), false)
 	defer logger.Close()
 
 	logger.Println("Running with", AppVersion.Line())
