@@ -3,6 +3,7 @@
 package docker
 
 import (
+	"bufio"
 	"context"
 	"errors"
 	"fmt"
@@ -225,7 +226,7 @@ func TestHelperImageWithVariable(t *testing.T) {
 	runnerImageTag := "gitlab/gitlab-runner:" + common.AppVersion.Revision
 
 	p.On("GetDockerImage", runnerImageTag, common.ImageDockerOptions{}, []common.DockerPullPolicy(nil)).
-		Return(&types.ImageInspect{ID: "helper-image"}, nil).
+		Return(&image.InspectResponse{ID: "helper-image"}, nil).
 		Once()
 
 	e := executorWithMockClient(c)
@@ -754,7 +755,7 @@ func TestCreateDependencies(t *testing.T) {
 			})
 
 			c.On("ImageInspectWithRaw", mock.Anything, "alpine:latest").
-				Return(types.ImageInspect{}, nil, nil).
+				Return(image.InspectResponse{}, nil, nil).
 				Once()
 			c.On("NetworkList", mock.Anything, mock.Anything).
 				Return(nil, nil).
@@ -858,7 +859,7 @@ func prepareTestDockerConfiguration(
 	c, e := createExecutorForTestDockerConfiguration(t, dockerConfig, cce)
 
 	c.On("ImageInspectWithRaw", mock.Anything, expectedInspectImage).
-		Return(types.ImageInspect{ID: "123"}, []byte{}, nil).Twice()
+		Return(image.InspectResponse{ID: "123"}, []byte{}, nil).Twice()
 	c.On("ImagePullBlocking", mock.Anything, expectedPullImage, mock.Anything).
 		Return(nil).Once()
 	c.On("NetworkList", mock.Anything, mock.Anything).
@@ -876,7 +877,7 @@ func testDockerConfigurationWithJobContainer(
 ) {
 	c, e := prepareTestDockerConfiguration(t, dockerConfig, cce, "alpine", "alpine:latest")
 	c.On("ContainerInspect", mock.Anything, "abc").
-		Return(types.ContainerJSON{}, nil).Once()
+		Return(container.InspectResponse{}, nil).Once()
 
 	err := e.createVolumesManager()
 	require.NoError(t, err)
@@ -898,7 +899,7 @@ func testDockerConfigurationWithPredefinedContainer(
 	c, e := prepareTestDockerConfiguration(t, dockerConfig, cce, "alpine", "alpine:latest")
 
 	c.On("ContainerInspect", mock.Anything, "abc").
-		Return(types.ContainerJSON{}, nil).Once()
+		Return(container.InspectResponse{}, nil).Once()
 
 	err := e.createVolumesManager()
 	require.NoError(t, err)
@@ -2034,7 +2035,7 @@ func TestLocalHelperImage(t *testing.T) {
 		jobVariables     common.JobVariables
 		config           helperimage.Config
 		clientAssertions func(*docker.MockClient)
-		expectedImage    *types.ImageInspect
+		expectedImage    *image.InspectResponse
 	}{
 		"docker import using registry.gitlab.com name": {
 			config: helperimage.Config{
@@ -2055,7 +2056,7 @@ func TestLocalHelperImage(t *testing.T) {
 					},
 				).Return(nil)
 
-				imageInspect := types.ImageInspect{
+				imageInspect := image.InspectResponse{
 					RepoTags: []string{
 						imageName("", ""),
 					},
@@ -2067,7 +2068,7 @@ func TestLocalHelperImage(t *testing.T) {
 					imageName("", ""),
 				).Return(imageInspect, []byte{}, nil)
 			},
-			expectedImage: &types.ImageInspect{
+			expectedImage: &image.InspectResponse{
 				RepoTags: []string{
 					imageName("", ""),
 				},
@@ -2107,7 +2108,7 @@ func TestLocalHelperImage(t *testing.T) {
 					"ImageInspectWithRaw",
 					mock.Anything,
 					mock.Anything,
-				).Return(types.ImageInspect{}, []byte{}, errors.New("error"))
+				).Return(image.InspectResponse{}, []byte{}, errors.New("error"))
 			},
 			expectedImage: nil,
 		},
@@ -2133,7 +2134,7 @@ func TestLocalHelperImage(t *testing.T) {
 					mock.Anything,
 				).Return(nil)
 
-				imageInspect := types.ImageInspect{
+				imageInspect := image.InspectResponse{
 					RepoTags: []string{
 						imageName("", "-pwsh"),
 					},
@@ -2145,7 +2146,7 @@ func TestLocalHelperImage(t *testing.T) {
 					imageName("", "-pwsh"),
 				).Return(imageInspect, []byte{}, nil)
 			},
-			expectedImage: &types.ImageInspect{
+			expectedImage: &image.InspectResponse{
 				RepoTags: []string{
 					imageName("", "-pwsh"),
 				},
@@ -2174,7 +2175,7 @@ func TestLocalHelperImage(t *testing.T) {
 					mock.Anything,
 				).Return(nil)
 
-				imageInspect := types.ImageInspect{
+				imageInspect := image.InspectResponse{
 					RepoTags: []string{
 						imageName("ubuntu-", "-pwsh"),
 					},
@@ -2186,7 +2187,7 @@ func TestLocalHelperImage(t *testing.T) {
 					imageName("ubuntu-", "-pwsh"),
 				).Return(imageInspect, []byte{}, nil)
 			},
-			expectedImage: &types.ImageInspect{
+			expectedImage: &image.InspectResponse{
 				RepoTags: []string{
 					imageName("ubuntu-", "-pwsh"),
 				},
@@ -2213,7 +2214,7 @@ func TestLocalHelperImage(t *testing.T) {
 					imageName("ubuntu-", ""),
 				).Return(nil)
 
-				imageInspect := types.ImageInspect{
+				imageInspect := image.InspectResponse{
 					RepoTags: []string{
 						imageName("ubuntu-", ""),
 					},
@@ -2225,7 +2226,7 @@ func TestLocalHelperImage(t *testing.T) {
 					imageName("ubuntu-", ""),
 				).Return(imageInspect, []byte{}, nil)
 			},
-			expectedImage: &types.ImageInspect{
+			expectedImage: &image.InspectResponse{
 				RepoTags: []string{
 					imageName("ubuntu-", ""),
 				},
@@ -2366,7 +2367,7 @@ func TestExpandingDockerImageWithImagePullPolicyAlways(t *testing.T) {
 	c, e := prepareTestDockerConfiguration(t, dockerConfig, cce, "alpine", "alpine:latest")
 
 	c.On("ContainerInspect", mock.Anything, "abc").
-		Return(types.ContainerJSON{}, nil).Once()
+		Return(container.InspectResponse{}, nil).Once()
 
 	err := e.createVolumesManager()
 	require.NoError(t, err)
@@ -2552,6 +2553,7 @@ func TestDockerImageWithUser(t *testing.T) {
 		"ok allowed users, runner user specified": {allowedUsers: []string{"baba"}, runnerUser: "baba", want: "baba"},
 		"ok allowed users, job user specified":    {allowedUsers: []string{"baba"}, jobUser: "baba", want: "baba"},
 		"ok allowed users, both specified":        {allowedUsers: []string{"baba"}, runnerUser: "baba", jobUser: "yaga", want: "baba"},
+		"ok allowed users, job user as variable":  {allowedUsers: []string{"baba"}, jobUser: "${TTUSER}", want: "baba"},
 
 		"bad allowed users, runner user specified": {allowedUsers: []string{"yaga"}, runnerUser: "baba", want: "", wantErr: true},
 		"bad allowed users, job user specified":    {allowedUsers: []string{"yaga"}, jobUser: "baba", want: "", wantErr: true},
@@ -2579,7 +2581,7 @@ func TestDockerImageWithUser(t *testing.T) {
 
 			c, e := createExecutorForTestDockerConfiguration(t, dockerConfig, cce)
 			c.On("ImageInspectWithRaw", mock.Anything, mock.Anything).
-				Return(types.ImageInspect{ID: "123"}, []byte{}, nil).Maybe()
+				Return(image.InspectResponse{ID: "123"}, []byte{}, nil).Maybe()
 			c.On("ImagePullBlocking", mock.Anything, mock.Anything, mock.Anything).
 				Return(nil).Maybe()
 			c.On("NetworkList", mock.Anything, mock.Anything).
@@ -2587,7 +2589,12 @@ func TestDockerImageWithUser(t *testing.T) {
 			c.On("ContainerRemove", mock.Anything, mock.Anything, mock.Anything).
 				Return(nil).Maybe()
 			c.On("ContainerInspect", mock.Anything, "abc").
-				Return(types.ContainerJSON{}, nil).Maybe()
+				Return(container.InspectResponse{}, nil).Maybe()
+
+			e.Build.Variables = append(e.Build.Variables, common.JobVariable{
+				Key:   "TTUSER",
+				Value: tt.want,
+			})
 
 			err := e.createVolumesManager()
 			require.NoError(t, err)
@@ -2606,26 +2613,6 @@ func TestDockerImageWithUser(t *testing.T) {
 	}
 }
 
-func TestCreateHostConfigForServiceHealthCheck(t *testing.T) {
-	tests := []string{"default", "user-defined"}
-
-	for _, tc := range tests {
-		t.Run("network "+tc, func(t *testing.T) {
-			e := &executor{
-				networkMode: container.NetworkMode(tc),
-			}
-
-			hostConfig := e.createHostConfigForServiceHealthCheck(&types.Container{Names: []string{"service-name"}})
-
-			if e.networkMode.UserDefined() != "" {
-				require.Empty(t, hostConfig.Links)
-			} else {
-				require.NotEmpty(t, hostConfig.Links)
-			}
-		})
-	}
-}
-
 var _ executors.Environment = (*env)(nil)
 
 type env struct {
@@ -2746,18 +2733,18 @@ func TestTooManyServicesRequestedError(t *testing.T) {
 	})
 }
 
-func Test_createStepRunnerVolume(t *testing.T) {
+func Test_bootstrap(t *testing.T) {
 	type testCase struct {
-		setup         func(*volumes.MockManager, *common.Build) []string
+		setup         func(*volumes.MockManager, *docker.MockClient, *common.Build) []string
 		expectedBinds []string
 		wantStage     common.ExecutorStage
 	}
 	tests := map[string]map[string]testCase{
 		"linux": {
 			"native steps enabled": {
-				expectedBinds: []string{"/opt/step-runner"},
-				wantStage:     ExecutorStageCreatingStepRunnerVolume,
-				setup: func(vm *volumes.MockManager, b *common.Build) []string {
+				expectedBinds: []string{"/opt/gitlab-runner"},
+				wantStage:     ExecutorStageBootstrap,
+				setup: func(vm *volumes.MockManager, c *docker.MockClient, b *common.Build) []string {
 					binds := make([]string, 1)
 					b.JobResponse.Run = "blablabla"
 					b.Variables = append(b.Variables, common.JobVariable{
@@ -2765,12 +2752,42 @@ func Test_createStepRunnerVolume(t *testing.T) {
 						Value: "true",
 					})
 
-					vm.On("CreateTemporary", mock.Anything, "/opt/step-runner").
+					c.EXPECT().ImageInspectWithRaw(mock.Anything, mock.Anything).Return(image.InspectResponse{
+						ID: "helper-id",
+					}, nil, nil)
+					c.EXPECT().ContainerCreate(mock.Anything, &container.Config{
+						Image:           "helper-id",
+						Cmd:             []string{"gitlab-runner-helper", "steps", "bootstrap", bootstrappedBinary},
+						Tty:             false,
+						AttachStdin:     false,
+						AttachStdout:    true,
+						AttachStderr:    true,
+						OpenStdin:       false,
+						StdinOnce:       true,
+						NetworkDisabled: true,
+					}, mock.Anything, mock.Anything, mock.Anything, mock.Anything).Return(container.CreateResponse{ID: "container-id"}, nil)
+
+					c.EXPECT().ContainerAttach(mock.Anything, "container-id", mock.Anything).Return(types.HijackedResponse{
+						Reader: bufio.NewReader(strings.NewReader("")),
+						Conn:   &net.UnixConn{},
+					}, nil)
+					c.EXPECT().ContainerRemove(mock.Anything, "container-id", mock.Anything).Return(nil)
+
+					bodyCh := make(chan container.WaitResponse, 1)
+					bodyCh <- container.WaitResponse{StatusCode: 0}
+					c.EXPECT().ContainerWait(mock.Anything, "container-id", container.WaitConditionNextExit).
+						Return((<-chan container.WaitResponse)(bodyCh), nil)
+
+					c.EXPECT().ContainerStart(mock.Anything, "container-id", mock.Anything).Return(nil)
+
+					vm.EXPECT().CreateTemporary(mock.Anything, "/opt/gitlab-runner").
 						Return(nil).
-						Run(func(args mock.Arguments) {
-							binds[0] = args.Get(1).(string)
+						Run(func(ctx context.Context, destination string) {
+							binds[0] = destination
 						}).
 						Once()
+					vm.EXPECT().Binds().Return(binds).Once()
+
 					return binds
 				},
 			},
@@ -2784,26 +2801,226 @@ func Test_createStepRunnerVolume(t *testing.T) {
 
 	for name, tt := range tests[runtime.GOOS] {
 		t.Run(name, func(t *testing.T) {
+			c := docker.NewMockClient(t)
 			vm := volumes.NewMockManager(t)
 			e := executor{
 				volumesManager: vm,
+				dockerConn:     &dockerConnection{Client: c},
 				AbstractExecutor: executors.AbstractExecutor{
+					Context: t.Context(),
 					Build: &common.Build{
 						ExecutorFeatures: common.FeaturesInfo{
 							NativeStepsIntegration: true,
 						},
 					},
+					Config: common.RunnerConfig{
+						RunnerSettings: common.RunnerSettings{
+							Docker: &common.DockerConfig{},
+						},
+					},
 				},
 			}
 
 			var binds []string
 			if tt.setup != nil {
-				binds = tt.setup(vm, e.Build)
+				binds = tt.setup(vm, c, e.Build)
 			}
 
-			assert.NoError(t, e.createStepRunnerVolume())
+			assert.NoError(t, e.bootstrap())
 			assert.Equal(t, tt.expectedBinds, binds)
 			assert.Equal(t, tt.wantStage, e.GetCurrentStage())
 		})
 	}
 }
+
+// TestDockerSlotCgroupSettings verifies that slot-based cgroup settings
+// are actually applied to container HostConfig when creating containers
+func TestDockerSlotCgroupSettings(t *testing.T) {
+	t.Run("Build container with slot cgroups enabled", func(t *testing.T) {
+		runnerConfig := &common.RunnerConfig{
+			RunnerSettings: common.RunnerSettings{
+				UseSlotCgroups:     true,
+				SlotCgroupTemplate: "runner/slot-${slot}",
+				Docker: &common.DockerConfig{
+					CgroupParent: "should-not-use-this",
+				},
+			},
+		}
+
+		// Verify HostConfig.CgroupParent is set to slot-based value
+		cce := func(t *testing.T, config *container.Config, hostConfig *container.HostConfig, _ *network.NetworkingConfig) {
+			assert.Equal(t, "runner/slot-5", hostConfig.CgroupParent, "HostConfig.CgroupParent should be set to slot-based value")
+		}
+
+		testDockerConfigurationWithSlotCgroups(t, runnerConfig, &mockAutoscalerExecutorData{slot: 5}, cce)
+	})
+
+	t.Run("Build container with slot cgroups enabled using default template", func(t *testing.T) {
+		runnerConfig := &common.RunnerConfig{
+			RunnerSettings: common.RunnerSettings{
+				UseSlotCgroups: true,
+				Docker: &common.DockerConfig{
+					CgroupParent: "fallback-cgroup",
+				},
+			},
+		}
+
+		cce := func(t *testing.T, config *container.Config, hostConfig *container.HostConfig, _ *network.NetworkingConfig) {
+			assert.Equal(t, "gitlab-runner/slot-10", hostConfig.CgroupParent, "HostConfig.CgroupParent should use default template")
+		}
+
+		testDockerConfigurationWithSlotCgroups(t, runnerConfig, &mockAutoscalerExecutorData{slot: 10}, cce)
+	})
+
+	t.Run("Build container with slot cgroups disabled", func(t *testing.T) {
+		runnerConfig := &common.RunnerConfig{
+			RunnerSettings: common.RunnerSettings{
+				UseSlotCgroups: false,
+				Docker: &common.DockerConfig{
+					CgroupParent: "static-build-cgroup",
+				},
+			},
+		}
+
+		cce := func(t *testing.T, config *container.Config, hostConfig *container.HostConfig, _ *network.NetworkingConfig) {
+			assert.Equal(t, "static-build-cgroup", hostConfig.CgroupParent, "HostConfig.CgroupParent should use static value when slot cgroups disabled")
+		}
+
+		testDockerConfigurationWithSlotCgroups(t, runnerConfig, &mockAutoscalerExecutorData{slot: 5}, cce)
+	})
+
+	t.Run("Build container with slot cgroups enabled but no slot available", func(t *testing.T) {
+		runnerConfig := &common.RunnerConfig{
+			RunnerSettings: common.RunnerSettings{
+				UseSlotCgroups: true,
+				Docker: &common.DockerConfig{
+					CgroupParent: "fallback-build-cgroup",
+				},
+			},
+		}
+
+		cce := func(t *testing.T, config *container.Config, hostConfig *container.HostConfig, _ *network.NetworkingConfig) {
+			assert.Equal(t, "fallback-build-cgroup", hostConfig.CgroupParent, "HostConfig.CgroupParent should fallback when no slot available")
+		}
+
+		testDockerConfigurationWithSlotCgroups(t, runnerConfig, nil, cce)
+	})
+
+	t.Run("Service container with slot cgroups enabled", func(t *testing.T) {
+		runnerConfig := &common.RunnerConfig{
+			RunnerSettings: common.RunnerSettings{
+				UseSlotCgroups: true,
+				Docker: &common.DockerConfig{
+					ServiceCgroupParent:       "should-not-use-this",
+					ServiceSlotCgroupTemplate: "runner/service-${slot}",
+				},
+			},
+		}
+
+		testDockerServiceContainerCgroup(t, runnerConfig, &mockAutoscalerExecutorData{slot: 7}, "runner/service-7")
+	})
+
+	t.Run("Service container with slot cgroups enabled using default template", func(t *testing.T) {
+		runnerConfig := &common.RunnerConfig{
+			RunnerSettings: common.RunnerSettings{
+				UseSlotCgroups: true,
+				Docker: &common.DockerConfig{
+					ServiceCgroupParent: "fallback-service",
+				},
+			},
+		}
+
+		testDockerServiceContainerCgroup(t, runnerConfig, &mockAutoscalerExecutorData{slot: 3}, "gitlab-runner/slot-3")
+	})
+
+	t.Run("Service container with slot cgroups disabled", func(t *testing.T) {
+		runnerConfig := &common.RunnerConfig{
+			RunnerSettings: common.RunnerSettings{
+				UseSlotCgroups: false,
+				Docker: &common.DockerConfig{
+					ServiceCgroupParent: "static-service-cgroup",
+				},
+			},
+		}
+
+		testDockerServiceContainerCgroup(t, runnerConfig, &mockAutoscalerExecutorData{slot: 5}, "static-service-cgroup")
+	})
+
+	t.Run("Service container with slot cgroups enabled but no slot available", func(t *testing.T) {
+		runnerConfig := &common.RunnerConfig{
+			RunnerSettings: common.RunnerSettings{
+				UseSlotCgroups: true,
+				Docker: &common.DockerConfig{
+					ServiceCgroupParent: "fallback-service-cgroup",
+				},
+			},
+		}
+
+		testDockerServiceContainerCgroup(t, runnerConfig, nil, "fallback-service-cgroup")
+	})
+}
+
+// Mock ExecutorData for testing slot functionality
+type mockAutoscalerExecutorData struct {
+	slot int
+}
+
+func (m *mockAutoscalerExecutorData) AcquisitionSlot() int {
+	return m.slot
+}
+
+// testDockerConfigurationWithSlotCgroups tests that build containers are created with slot-based cgroups
+func testDockerConfigurationWithSlotCgroups(
+	t *testing.T,
+	runnerConfig *common.RunnerConfig,
+	executorData interface{},
+	cce containerConfigExpectations,
+) {
+	c, e := prepareTestDockerConfiguration(t, runnerConfig.Docker, cce, "alpine", "alpine:latest")
+	c.On("ContainerInspect", mock.Anything, "abc").
+		Return(container.InspectResponse{}, nil).Once()
+
+	// Set the executor data for slot testing
+	e.Build.ExecutorData = executorData
+	// Set the runner config for slot testing
+	e.Config = *runnerConfig
+
+	err := e.createVolumesManager()
+	require.NoError(t, err)
+
+	err = e.createPullManager()
+	require.NoError(t, err)
+
+	imageConfig := common.Image{Name: "alpine"}
+	cfgTor := newDefaultContainerConfigurator(e, buildContainerType, imageConfig, []string{"/bin/sh"}, []string{})
+	_, err = e.createContainer(buildContainerType, imageConfig, []string{}, cfgTor)
+	assert.NoError(t, err, "Should create container without errors")
+}
+
+// testDockerServiceContainerCgroup tests that service containers are created with the expected cgroup parent
+func testDockerServiceContainerCgroup(
+	t *testing.T,
+	runnerConfig *common.RunnerConfig,
+	executorData interface{},
+	expectedCgroup string,
+) {
+	// Create mock docker client
+	c := docker.NewMockClient(t)
+
+	// Create mock volumes manager
+	vm := volumes.NewMockManager(t)
+	vm.On("Binds").Return([]string{})
+
+	e := new(executor)
+	e.dockerConn = &dockerConnection{Client: c}
+	e.Config = *runnerConfig
+	e.Build = &common.Build{
+		ExecutorData: executorData,
+	}
+	e.volumesManager = vm
+
+	// Call createHostConfigForService and verify the cgroup is set correctly
+	hostConfig, err := e.createHostConfigForService(false, nil, nil)
+	require.NoError(t, err)
+	assert.Equal(t, expectedCgroup, hostConfig.CgroupParent, "Service container HostConfig.CgroupParent should be set correctly")
+}
