@@ -51,6 +51,7 @@ const (
 	UseAdaptiveRequestConcurrency        string = "FF_USE_ADAPTIVE_REQUEST_CONCURRENCY"
 	UseGitalyCorrelationId               string = "FF_USE_GITALY_CORRELATION_ID"
 	HashCacheKeys                        string = "FF_HASH_CACHE_KEYS"
+	EnableJobInputsInterpolation         string = "FF_ENABLE_JOB_INPUTS_INTERPOLATION"
 )
 
 type FeatureFlag struct {
@@ -326,9 +327,9 @@ var flags = []FeatureFlag{
 	},
 	{
 		Name:         UseTimestamps,
-		DefaultValue: false,
+		DefaultValue: true,
 		Deprecated:   false,
-		Description:  "When enabled timestamps are added to the beginning of each log trace line.",
+		Description:  "When disabled timestamps are not added to the beginning of each log trace line.",
 	},
 	{
 		Name:         DisableAutomaticTokenRotation,
@@ -422,6 +423,12 @@ var flags = []FeatureFlag{
 		Description: "When GitLab Runner creates or extracts caches, it hashes the cache keys (SHA256) before using them, both for local " +
 			"and distributed caches (for example, S3). For more information, see [cache key handling](advanced-configuration.md#cache-key-handling).",
 	},
+	{
+		Name:         EnableJobInputsInterpolation,
+		DefaultValue: false,
+		Deprecated:   false,
+		Description:  "When enabled, job inputs are interpolated. For more information, see [&17833](https://gitlab.com/groups/gitlab-org/-/epics/17833).",
+	},
 }
 
 func GetAll() []FeatureFlag {
