@@ -838,7 +838,7 @@ func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, info common.ShellScr
 	w.Command("git", "remote", "set-url", "origin", remoteURL)
 	// For existing repositories, the template isn't reapplied, so we need to explicitly
 	// configure the repository to use the external git config
-	b.setupExistingRepoConfig(w, projectDir)
+	b.setupExistingRepoConfig(w)
 	w.EndIf()
 
 	v := common.AppVersion
@@ -934,10 +934,16 @@ func includeExternalGitConfig(w ShellWriter, targetFile, fileToInclude string) {
 
 // setupExistingRepoConfig configures an existing Git repository to use the external Git config.
 // This is needed because git init with a template doesn't re-apply the template to existing repositories.
-func (b *AbstractShell) setupExistingRepoConfig(w ShellWriter, projectDir string) {
-	extConfigFile := w.TmpFile(externalGitConfigFile)
-	gitConfigFile := w.Join(projectDir, gitDir, "config")
-	includeExternalGitConfig(w, gitConfigFile, extConfigFile)
+// Note: This function relies on the GLR_EXT_GIT_CONFIG_PATH environment variable being set by setupTemplateDir().
+func (b *AbstractShell) setupExistingRepoConfig(w ShellWriter) {
+	// We're already in projectDir (after cd), so .git/config is relative to current directory
+	gitConfigFile := w.Join(gitDir, "config")
+	// Use the environment variable that was set in setupTemplateDir() to get the absolute path.
+	// This ensures the path is correct even when BuildDir is relative and we've already changed directories.
+	extConfigFile := w.EnvVariableKey(envVarExternalGitConfigFile)
+	baseName := path.Base(helpers.ToSlash(externalGitConfigFile))
+	pattern := regexp.QuoteMeta(baseName) + "$"
+	w.CommandArgExpand("git", "config", "--file", gitConfigFile, "--replace-all", "include.path", extConfigFile, pattern)
 }
 
 func (b *AbstractShell) setupTemplateDir(w ShellWriter, build *common.Build, projectDir string) (string, string, error) {
@@ -1154,8 +1160,17 @@ func (b *AbstractShell) writeSubmoduleUpdateCmd(w ShellWriter, build *common.Bui
 
 	w.Command("git", append(foreachArgs, cleanCommand...)...)
 
+	// Configure each submodule to include the external git config.
+	// This allows git operations inside submodule directories (e.g., cd patches && git pull)
+	// to authenticate properly using the parent repo's credentials.
+	// This is done once at the end, after all submodule operations are complete.
+	// See: https://gitlab.com/gitlab-org/gitlab-runner/-/issues/39133
+	w.Noticef("Configuring submodules to use parent git credentials...")
+	b.configureSubmoduleCredentials(w, foreachArgs, recursive)
+
 	if !build.IsLFSSmudgeDisabled() {
 		w.IfCmd("git", "lfs", "version")
+		w.Noticef("Pulling LFS files...")
 		w.CommandArgExpand("git", withExplicitSubmoduleCreds(append(foreachArgs, "git lfs pull"))...)
 		w.EndIf()
 	}
@@ -1176,6 +1191,23 @@ func (b *AbstractShell) writeSubmoduleUpdateNoticeMsg(w ShellWriter, recursive b
 	}
 }
 
+// configureSubmoduleCredentials configures each submodule to include the external git config
+// from the parent repository. This allows git operations inside submodule directories
+// (e.g., cd patches && git pull) to authenticate properly using the parent repo's credentials.
+func (b *AbstractShell) configureSubmoduleCredentials(w ShellWriter, foreachArgs []string, recursive bool) {
+	// Use the GLR_EXT_GIT_CONFIG_PATH environment variable that was set earlier.
+	// We need to quote the variable expansion to handle paths with spaces.
+	cmd := fmt.Sprintf(`git config --replace-all include.path '%s'`, w.EnvVariableKey(envVarExternalGitConfigFile))
+	args := foreachArgs
+	// Even if `GIT_SUBMODULE_STRATEGY: normal` is used, we should set up the credentials
+	// for all the Git submodules to preserve existing workflows.
+	if !recursive {
+		args = append(args, "--recursive")
+	}
+	args = append(args, cmd)
+	w.CommandArgExpand("git", args...)
+}
+
 func (b *AbstractShell) writeRestoreCacheScript(
 	ctx context.Context,
 	w ShellWriter,
